@@ -26,10 +26,10 @@ Enables identical use for installed and uninstalled versions.
 import os.path
 import string
 
-LIBDIR = '/usr/lib'
-PKGDATADIR = '/usr/share/pitivi'
-PIXMAPDIR = '/usr/share/pixmaps'
-pitivi_version = '0.11.1.4'
+LIBDIR = '/src/lib'
+PKGDATADIR = '/src/share/pitivi'
+PIXMAPDIR = '/src/share/pixmaps'
+pitivi_version = '0.11.2'
 APPNAME = 'PiTiVi'
 PYGTK_REQ = '2.8.0'
 PYGST_REQ = '0.10.6'
