@@ -80,10 +80,10 @@ class PlayGround(gobject.GObject):
         gobject.GObject.__init__(self)
         # List of used pipelines
         self.pipelines = []
-        
+
         self.vsinkthread = None
         self.asinkthread = None
-        
+
         # Defaut pipeline if no other pipeline is playing
         self.default = SmartDefaultBin()
         bus = self.default.get_bus()
@@ -99,7 +99,7 @@ class PlayGround(gobject.GObject):
         self.tempsmartbin = None
         self.cur_state_signal = None
         self.cur_eos_signal = None
-        
+
         self.state = gst.STATE_READY
 
         # FIXME : THIS IS CRACK, THIS IS CRACK
@@ -118,7 +118,7 @@ class PlayGround(gobject.GObject):
         Adds the given pipeline to the playground.
         Returns True if the pipeline was added to the playground.
         """
-        gst.debug("pipeline : %s" % pipeline)
+        gst.debug("pipeline : %r" % pipeline)
         if not isinstance(pipeline, SmartBin):
             return False
 
@@ -135,13 +135,13 @@ class PlayGround(gobject.GObject):
         Removes the given pipeline from the playground.
         Return True if everything went well.
         """
-        gst.debug("pipeline : %s" % pipeline)
+        gst.debug("pipeline : %r" % pipeline)
         if not pipeline in self.pipelines:
             return True
 
         bus = pipeline.get_bus()
         bus.remove_signal_watch()
-        bus.set_sync_handler(None)
+        #bus.set_sync_handler(None)
 
         if pipeline.set_state(gst.STATE_READY) == gst.STATE_CHANGE_FAILURE:
             return False
@@ -154,7 +154,7 @@ class PlayGround(gobject.GObject):
 
     def switchToPipeline(self, pipeline):
         """
-        Switch to the given pipeline for play output.        
+        Switch to the given pipeline for play output.
         Returns True if the switch was possible.
         """
         if self.current == pipeline:
@@ -206,17 +206,22 @@ class PlayGround(gobject.GObject):
         will be taken.
         """
         if isinstance(self.current, SmartTimelineBin):
-            # fast path
             return True
+        p = self.getTimeline()
+        if p:
+            self.switchToPipeline(p)
+            return True
+        return False
+
+    def getTimeline(self):
         for pipeline in self.pipelines:
             if isinstance(pipeline, SmartTimelineBin):
-                self.switchToPipeline(pipeline)
-                return True
-        return False
+                return pipeline
+        return None
 
     def setVideoSinkThread(self, vsinkthread):
         """ sets the video sink thread """
-        gst.debug("video sink thread : %s" % vsinkthread)
+        gst.debug("video sink thread : %r" % vsinkthread)
         if self.vsinkthread and self.current.has_video:
             self.current.set_state(gst.STATE_READY)
             self.current.removeVideoSinkThread()
@@ -226,7 +231,7 @@ class PlayGround(gobject.GObject):
 
     def setAudioSinkThread(self, asinkthread):
         """ sets the audio sink thread """
-        gst.debug("set audio sink thread : %s" % asinkthread)
+        gst.debug("set audio sink thread : %r" % asinkthread)
         if self.asinkthread and self.current.asinkthread:
             self.current.set_state(gst.STATE_READY)
             self.current.removeAudioSinkThread()
@@ -234,41 +239,56 @@ class PlayGround(gobject.GObject):
         if self.current and self.current.has_audio:
             self.current.setAudioSinkThread(self.asinkthread)
 
-    def _playTemporaryBin(self, tempbin):
+    def _playTemporaryBin(self, tempbin, playit=True):
         """
         Temporarely play a smartbin.
         Return False if there was a problem.
+
+        By default the bin will start playing, except if you set
+        playit to False
         """
-        gst.debug("BEGINNING tempbin : %s" % tempbin)
+        gst.debug("BEGINNING tempbin : %r" % tempbin)
         self.pause()
         self.addPipeline(tempbin)
         self.switchToPipeline(tempbin)
         if self.tempsmartbin:
             self.removePipeline(self.tempsmartbin)
         self.tempsmartbin = tempbin
-        if self.play() == gst.STATE_CHANGE_FAILURE:
+        if playit and self.play() == gst.STATE_CHANGE_FAILURE:
             return False
-        gst.debug("END tempbin : %s" % tempbin)
+        gst.debug("END tempbin : %r" % tempbin)
         return True
 
-    def playTemporaryFilesourcefactory(self, factory):
+    def playTemporaryFilesourcefactory(self, factory, playit=True):
         """
         Temporarely play a FileSourceFactory.
         Returns False if there was a problem.
+
+        By default the bin will start playing, except if you set
+        playit to False
         """
         gst.debug("factory : %s" % factory)
         if isinstance(self.current, SmartFileBin) and self.current.factory == factory:
             gst.info("Already playing factory : %s" % factory)
             return True
         tempbin = SmartFileBin(factory)
-        return self._playTemporaryBin(tempbin)
+        return self._playTemporaryBin(tempbin, playit)
 
-    def getCurrentTimePosition(self):
+    def getCurrentTimePosition(self, onlyvideo=True):
         """
         Returns the current position of the current bin in gst.FORMAT_TIME
         """
+        # FIXME : Yes, we're forcing by default to query the position only on
+        # the video stream. The problem is that because of some formats screwing
+        # up big time when doing accurate seek (namely mad/mp3 in avi), the
+        # position in the audiosink will be greater than the position in the
+        # videosink... and the result will be a position further than what's
+        # actually being displayed currently :(
         try:
-            cur, format = self.current.query_position(gst.FORMAT_TIME)
+            if onlyvideo and not self.current.recording:
+                cur,format = self.vsinkthread.query_position(gst.FORMAT_TIME)
+            else:
+                cur, format = self.current.query_position(gst.FORMAT_TIME)
         except:
             self.current.warning("Couldn't get position in time")
             cur = 0
@@ -305,7 +325,7 @@ class PlayGround(gobject.GObject):
         gst.debug("Seeking to %s succeeded" % gst.TIME_ARGS (value))
 
         self.emit('position', self.current, value)
-        
+
         return True
 
     def shutdown(self):
@@ -340,7 +360,7 @@ class PlayGround(gobject.GObject):
     #
     def _busMessageCb(self, unused_bus, message, unused_pipeline):
         """ handler for messages from the pipelines' buses """
-        gst.info("%s [%s]" % (message.type, message.src))
+        gst.info("%s [%r]" % (message.type, message.src))
         if message.type == gst.MESSAGE_STATE_CHANGED:
             oldstate, newstate, pending = message.parse_state_changed()
             message.src.debug("old:%s, new:%s, pending:%s" %
@@ -348,7 +368,7 @@ class PlayGround(gobject.GObject):
             if (not self.current == self.default) and message.src == self.current:
                 if pending == gst.STATE_VOID_PENDING:
                     if (not self._positiontimeoutid) and newstate in [gst.STATE_PLAYING]:
-			self._checkTimeCb()
+                        self._checkTimeCb()
                         self._positiontimeoutid = gobject.timeout_add(300, self._checkTimeCb)
                     self.emit("current-state", newstate)
         elif message.type == gst.MESSAGE_ERROR:
@@ -393,17 +413,17 @@ class PlayGround(gobject.GObject):
                           _("There is a problem with your audio output device"))
         else:
             self.emit("error", gerror.message, detail)
-            
-        
+
+
 
     def _handleWarning(self, gerror, detail, source):
         """
         Uses the information from the Gerror, detail and source to
         create meaningful warning messages for the User Interface.
         """
-        gst.warning("gerror:%s , detail:%s , source:%s" % (gerror, detail, source))
+        gst.warning("gerror:%s , detail:%s , source:%r" % (gerror, detail, source))
         gst.warning("GError : code:%s, domain:%s, message:%s" % (gerror.code, gerror.domain, gerror.message))
-        
+
 
     #
     # playing proxy functions
@@ -422,7 +442,7 @@ class PlayGround(gobject.GObject):
         ret = self.current.set_state(self.state)
         gst.debug("change state returned %s" % ret)
         return ret
- 
+
     def pause(self):
         """
         Set the current Pipeline to Pause.
@@ -447,15 +467,33 @@ class PlayGround(gobject.GObject):
         """ play the current pipeline backwards """
         pass
 
+    ## Frame by frame seeking
+    # Goes forward/backward in the stream by one video frame
+    # For timelines, it will use the current export setting framerate as the
+    # framerate to use for calculatiosn. Not perfect, but still very close.
+
     def forward_one(self):
         """ forward the current pipeline by one video frame """
-        pass
+        if not self.current or not type(self.current) in [SmartTimelineBin, SmartFileBin]:
+            gst.warning("you can only do frame seeking on files or timelines")
+            return
+        pos = self.getCurrentTimePosition()
+        framerate = self.current.getSettings().videorate
+        target = pos + framerate.denom * gst.SECOND / framerate.num
+        gst.log("current position is %s , seeking to %s" % (
+            gst.TIME_ARGS(pos),
+            gst.TIME_ARGS(target)))
+        self.seekInCurrent(target)
 
     def backward_one(self):
         """ rewind the current pipeline by one video frame """
-        pass
-
-    def seek(self, time):
-        """ seek in the current pipeline """
-        pass
-
+        if not self.current or not type(self.current) in [SmartTimelineBin, SmartFileBin]:
+            gst.warning("you can only do frame seeking on files or timelines")
+            return
+        pos = self.getCurrentTimePosition()
+        framerate = self.current.getSettings().videorate
+        target = pos - framerate.denom * gst.SECOND / framerate.num
+        gst.log("current position is %s , seeking to %s" % (
+            gst.TIME_ARGS(pos),
+            gst.TIME_ARGS(target)))
+        self.seekInCurrent(target)
