@@ -32,7 +32,6 @@ from sourcelist import SourceList
 from bin import SmartTimelineBin
 from settings import ExportSettings
 from configure import APPNAME
-
 from gettext import gettext as _
 from serializable import Serializable, to_object_from_data_type
 from projectsaver import ProjectSaver, ProjectSaveError, ProjectLoadError
@@ -41,10 +40,12 @@ class Project(gobject.GObject, Serializable):
     """ The base class for PiTiVi projects
     Signals
 
-       string save-uri-requested()
+       boolean save-uri-requested()
             The the current project has been requested to save itself, but
-            needs a URI to which to save. Handlers should return a uri to save
-            the file to or None, in which case the file will not be saved.
+            needs a URI to which to save. Handlers should first call
+            setUri(), with the uri to save the file (optionally
+            specifying the file format) and return True, or simply
+            return False to cancel the file save operation.
 
         boolean confirm-overwrite()
             The project has been requested to save itself, but the file on
@@ -60,7 +61,7 @@ class Project(gobject.GObject, Serializable):
 
     __gsignals__ = {
         "save-uri-requested" : ( gobject.SIGNAL_RUN_LAST,
-                                     gobject.TYPE_STRING,
+                                     gobject.TYPE_BOOLEAN,
                                      ( )),
         "confirm-overwrite" : ( gobject.SIGNAL_RUN_LAST,
                                 gobject.TYPE_BOOLEAN,
@@ -72,7 +73,7 @@ class Project(gobject.GObject, Serializable):
 
     __data_type__ = "project"
 
-    def __init__(self, name="", uri=None):
+    def __init__(self, name="", uri=None, **kwargs):
         """
         name : the name of the project
         uri : the uri of the project
@@ -83,6 +84,8 @@ class Project(gobject.GObject, Serializable):
         self.settings = None
         self.description = ""
         self.uri = uri
+        self.urichanged = False
+        self.format = None
         self.sources = SourceList(self)
         self.timeline = None
         self.timelinebin = None
@@ -90,35 +93,44 @@ class Project(gobject.GObject, Serializable):
         self._dirty = False
         self.timeline = Timeline(self)
         # don't want to make calling load() necessary for blank projects
-        if self.uri:
-            self._loaded = False
-        else:
+        if self.uri == None:
             self._loaded = True
-
+        else:
+            self._loaded = False
 
     def load(self):
         """ call this to load a project from a file (once) """
         if self._loaded:
-            # should this return false? 
+            # should this return false?
+            gst.warning("Already loaded !!!")
             return True
         return self._load()
-        
+
     def _load(self):
-        """ loads the project from a file """
-        if self.timeline:
-            return
-        self.timeline = Timeline(self)
-        if self.uri:
-            if file_is_project(self.uri):
-                loader = ProjectSaver.newProjectSaver("pickle")
-                path = gst.uri_get_location(self.uri)
-                fileobj = open(path, "w")
-                try:
-                    tree = loader.deserialize(fileobj)
-                    self.fromDataType(tree)
-                except ProjectLoadError:
-                    return False
+        """
+        loads the project from a file
+        Private method, use load() instead
+        """
+        gst.log("uri:%s" % self.uri)
+        gst.debug("Creating timeline")
+        # FIXME : This should be discovered !
+        saveformat = "pickle"
+        if self.uri and file_is_project(self.uri):
+            loader = ProjectSaver.newProjectSaver(saveformat)
+            path = gst.uri_get_location(self.uri)
+            fileobj = open(path, "r")
+            try:
+                tree = loader.openFromFile(fileobj)
+                self.fromDataFormat(tree)
+            except ProjectLoadError:
+                gst.error("Error while loading the project !!!")
+                return False
+            finally:
                 fileobj.close()
+            self.format = saveformat
+            self.urichanged = False
+            return True
+        return False
 
     def getBin(self):
         """ returns the SmartTimelineBin of the project """
@@ -133,40 +145,70 @@ class Project(gobject.GObject, Serializable):
         if uri_is_valid(self.uri):
             path = gst.uri_get_location(self.uri)
         else:
+            gst.warning("uri '%s' is invalid, aborting save" % self.uri)
             return False
+
         #TODO: a bit more sophisticated overwite detection
-        if os.path.exists(path):
+        if os.path.exists(path) and self.urichanged:
             if not self.emit("confirm-overwrite", self.uri):
+                gst.log("aborting save because overwrite was denied")
                 return False
+
         try:
-            # dummy save code for now
-            os.system("touch %s" % path)
+            fileobj = open(path, "w")
+            loader = ProjectSaver.newProjectSaver(self.format)
+            tree = self.toDataFormat()
+            loader.saveToFile(tree, fileobj)
             self._dirty = False
+            self.urichanged = False
+            gst.log("Project file saved successfully !")
             return True
         except IOError:
             return False
 
     def save(self):
         """ Saves the project to the project's current file """
-        if not self.uri:
-            uri = self.emit("save-uri-requested")
-            if not uri:
-                return False
-            self.uri = uri
-        return self._save()
+        gst.log("saving...")
+        if self.uri:
+            return self._save()
+
+        gst.log("requesting for a uri to save to...")
+        if self.emit("save-uri-requested"):
+            gst.log("'save-uri-requested' returned True, self.uri:%s" % self.uri)
+            if self.uri:
+                return self._save()
+
+        gst.log("'save-uri-requested' returned False or uri wasn't set, aborting save")
+        return False
 
     def saveAs(self):
         """ Saves the project to the given file name """
-        uri = self.emit("save-uri-requested")
-        if not uri:
+        if not self.emit("save-uri-requested"):
+            return False
+        if not self.uri:
             return False
-        self.uri = uri
         return self._save()
 
     # setting methods
     def _settingsChangedCb(self, unused_settings):
         self.emit('settings-changed')
 
+    def setUri(self, uri, format=None):
+        gst.log("uri:%s, format:%s" % (uri, format))
+        if not self.uri == uri:
+            gst.log("updating self.uri, previously:%s" % self.uri)
+            self.uri = uri
+            self.urichanged = True
+
+        if not format or not self.format == format:
+            gst.log("updating save format, previously:%s" % self.format)
+            if not format:
+                path = gst.uri_get_location(uri)
+                ext = os.path.splitext(path)[1]
+                gst.log("Based on file extension, format is %s" % format)
+                format = ProjectSaver.getFormat(ext)
+            self.format = format
+
     def getSettings(self):
         """
         return the currently configured settings.
@@ -220,6 +262,13 @@ class Project(gobject.GObject, Serializable):
     def hasUnsavedModifications(self):
         return self._dirty
 
+    # signals default handlers
+    def do_save_uri_requested(self):
+        return True
+
+    def do_confirm_overwrite(self, unused_uri):
+        return True
+
     # Serializable methods
 
     def toDataFormat(self):
@@ -233,9 +282,12 @@ class Project(gobject.GObject, Serializable):
     def fromDataFormat(self, obj):
         Serializable.fromDataFormat(self, obj)
         self.name = obj["name"]
-        self.timeline = to_object_from_data_type(obj["timeline"])
-        self.sources = to_object_from_data_type(obj["sources"])
-        self.settings = to_object_from_data_type(obj["settings"])
+        # calling this makes sure settigns-changed signal is emitted
+        self.setSettings(to_object_from_data_type(obj["settings"]))
+        # these objects already exist, so we initialize them from file
+        # to make sure UI catches signals
+        self.sources.fromDataFormat(obj["sources"])
+        self.timeline.fromDataFormat(obj["timeline"])
 
 def uri_is_valid(uri):
     return gst.uri_get_protocol(uri) == "file"
